from __future__ import absolute_import, print_function, unicode_literals

__name__ = 'pysyncgateway'
__version__ = '1.1.0'

__author__ = 'Construct Technology Ltd'
__copyright__ = '2018, {}'.format(__author__)

__description__ = 'Library for communication with Couchbase Sync Gateway'
__email__ = 'info@construct.pm'
